<?php

namespace Drupal\Tests\slick_browser\FunctionalJavascript;

use Drupal\Tests\entity_browser\FunctionalJavascript\EntityBrowserWebDriverTestBase;

/**
 * Tests the Slick Browser JavaScript using PhantomJS, or Chromedriver.
 *
 * @group slick_browser
 */
class SlickBrowserTest extends EntityBrowserWebDriverTestBase {

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'claro';

  /**
   * {@inheritdoc}
   */
  protected $testImage;

  /**
   * {@inheritdoc}
   */
  public static $modules = [
    'filter',
    'field',
    'user',
    'media',
    'inline_entity_form',
    'entity_browser',
    'entity_browser_entity_form',
    'entity_browser_test',
    'dropzonejs',
    'dropzonejs_eb_widget',
    'blazy',
    'blazy_test',
    'slick',
    'slick_browser',
    'slick_browser_test',
  ];

  /**
   * {@inheritdoc}
   */
  protected static $userPermissions = [
    'access slick_browser_file entity browser pages',
    'create article content',
    'access content',
    'create media',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp() {
    parent::setUp();

    /** @var \Drupal\Core\Entity\Display\EntityFormDisplayInterface $form_display */
    $form_display = $this->container->get('entity_type.manager')
      ->getStorage('entity_form_display')
      ->load('node.article.default');

    $settings = [
      'style' => 'grid',
      'image_style' => 'slick_browser_preview',
      'grid' => 3,
      'grid_medium' => 2,
      'grid_small' => 1,
    ];
    $form_display->setComponent('field_reference', [
      'type' => 'entity_browser_entity_reference',
      'settings' => [
        'entity_browser' => 'slick_browser_file',
        'field_widget_display' => 'slick_browser_file',
        'field_widget_remove' => TRUE,
        'field_widget_replace' => TRUE,
        'open' => TRUE,
        'selection_mode' => 'selection_append',
        // This is expected by file.
        'field_widget_display_settings' => $settings,
      ],
      // This is expected by image.
      'third_party_settings' => [
        'slick_browser' => $settings,
      ],
    ])->save();

    $this->testImage = $this->createFile('llama', 'jpg');
    $account = $this->drupalCreateUser(static::$userPermissions);
    $this->drupalLogin($account);
  }

  /**
   * Tests that selecting files in the view works even with direct selection.
   */
  public function testSlickBrowserFileDirectSelection() {
    $this->drupalGet('node/add/article');

    // Ensures Slick Widget exists.
    $this->assertSession()->elementExists('css', '.sb--widget');

    // Open the browser and select a file.
    $this->getSession()->switchToIFrame('entity_browser_iframe_slick_browser_file');

    $this->waitForAjaxToFinish();

    // Wait another moment, iframe build is slow.
    $this->assertSession()->elementNotExists('css', '.grid.is-checked');

    $this->assertSession()->elementExists('css', '.form--slick-browser-tabs');

    $this->assertCheckboxExistsByValue('file:' . $this->testImage->id());

    $this->getSession()->getPage()->find('css', '.grid--0')->press();

    $this->assertSession()->elementExists('css', '.grid--0.is-checked');

    // Ensures AJAX is triggered to insert the image into the page.
    $this->getSession()->getPage()->pressButton('Add to Page');

    // Switch back to the main page.
    $this->getSession()->switchToIFrame();
    $this->waitForAjaxToFinish();

    // Ensures image is inserted into the page.
    // @todo $this->assertSession()->pageTextContains('llama.jpg');
    $this->assertSession()->elementExists('css', '.sb__sortitem');

    // Tests the Delete functionality.
    $this->assertSession()->buttonExists('Remove');
    $this->getSession()->getPage()->find('css', '.button--wrap__mask')->press();
    $this->getSession()->getPage()->find('css', '.button--wrap__confirm')->press();
    $this->waitForAjaxToFinish();
    $this->assertSession()->pageTextNotContains('llama.jpg');
    $this->assertSession()->elementNotExists('css', '.sb__sortitem');
  }

  /**
   * Tests that selecting files in the view works even with delay selection.
   */
  public function testSlickBrowserFileDelaySelection() {
    $this->drupalGet('node/add/article');

    // Ensures Slick Widget exists.
    $this->assertSession()->elementExists('css', '.sb--widget');

    // Open the browser and select a file.
    $this->getSession()->switchToIFrame('entity_browser_iframe_slick_browser_file');

    $this->waitForAjaxToFinish();

    // Wait another moment, iframe build is slow.
    $this->assertSession()->elementNotExists('css', '.grid.is-checked');

    $this->assertSession()->elementExists('css', '.form--slick-browser-tabs');

    $this->assertCheckboxExistsByValue('file:' . $this->testImage->id());

    $this->getSession()->getPage()->find('css', '.grid--0')->press();

    $this->assertSession()->elementExists('css', '.grid--0.is-checked');

    // Delays selection.
    $this->getSession()->getPage()->pressButton('Select files');
    $this->waitForAjaxToFinish();

    // Ensures selected files were not gone.
    $this->assertSession()->elementExists('css', '.grid--0.was-checked');

    // Ensures AJAX is triggered to insert the image into the page.
    $this->getSession()->getPage()->pressButton('Add to Page');

    // Switch back to the main page.
    $this->getSession()->switchToIFrame();
    $this->waitForAjaxToFinish();

    // Ensures image is inserted into the page.
    $this->getSession()->wait(3000);

    // @todo $this->assertSession()->pageTextContains('llama.jpg');
    $this->assertSession()->elementExists('css', '.sb__sortitem');
  }

}
